import 'package:flutter/material.dart';
import 'package:google_ml_kit/google_ml_kit_for_korean.dart';

import 'camera_view.dart';
import 'painters/text_detector_painter.dart';

class TextDetectorView extends StatefulWidget {
  @override
  _TextDetectorViewState createState() => _TextDetectorViewState();
}

class _TextDetectorViewState extends State<TextDetectorView> {
  TextDetector textDetector = GoogleMlKit.vision.textDetector();
  bool isBusy = false;
  CustomPaint? customPaint;

  @override
  void dispose() async {
    super.dispose();
    await textDetector.close();
  }

  @override
  Widget build(BuildContext context) {
    return CameraView(
      title: 'Text Detector',
      customPaint: customPaint,
      onImage: (inputImage) {
        processImage(inputImage);
      },
    );
  }

  void getProductNameCandidate(recognisedText) {
    List<TextLine> productNameCandidates = [];
    for (TextBlock textBlock in recognisedText.blocks) {
      for (TextLine textLine in textBlock.lines) {
        String text = textLine.text;
        if (text.length < 10
            || text.startsWith(RegExp(r'[-.※]'))
            || RegExp(r"(^[\w]+$)").hasMatch(text)
            || text.startsWith('행사기간')) {
          continue;
        }
        print("Candidate for product name: " + text);
        productNameCandidates.add(textLine);
      }
    }

    if (productNameCandidates.isNotEmpty) {
      var longestTextLine = productNameCandidates.reduce(
              (current, next) => current.rect.right - current.rect.left > next.rect.right - next.rect.left ? current : next);
      print("===============> Final decision(product name): " + longestTextLine.text);
    }
  }

  void getPriceCandidate(recognisedText) {

  }

  void getBarcodeCandidate(recognisedText) {
    List<TextLine> barcodeCandidates = [];
    for (TextBlock textBlock in recognisedText.blocks) {
      for (TextLine textLine in textBlock.lines) {
        String text = textLine.text;

        if (text.length != 13) {
          continue;
        }

        if (RegExp(r"(^[0-9]+$)").hasMatch(text)
            || text.startsWith(RegExp(r'(200|880)'))) {

          print("Candidate for barcode: " + text);
          barcodeCandidates.add(textLine);
        }
      }
    }

    if (barcodeCandidates.isEmpty || barcodeCandidates.length > 1) {
      print('Invalid barcode. Try to take a new one.');
    } else {
      print("===============> Final decision(barcode): " + barcodeCandidates.first.text.replaceAll('[o|O]', '0'));
      // 공백제거 필요
    }
  }

  Future<void> processImage(InputImage inputImage) async {
    if (isBusy) return;
    isBusy = true;
    final recognisedText = await textDetector.processImage(inputImage);

    getProductNameCandidate(recognisedText);
    //getPriceCandidate(recognisedText);
    getBarcodeCandidate(recognisedText);

    print('Found ${recognisedText.blocks.length} textBlocks');
    if (inputImage.inputImageData?.size != null &&
        inputImage.inputImageData?.imageRotation != null) {
      final painter = TextDetectorPainter(
          recognisedText,
          inputImage.inputImageData!.size,
          inputImage.inputImageData!.imageRotation);
      customPaint = CustomPaint(painter: painter);
    } else {
      customPaint = null;
    }
    isBusy = false;
    if (mounted) {
      setState(() {});
    }
  }
}
